'use strict';

var util = require('util');
var fs = require('fs');
var path = require('path');

const statp = util.promisify(fs.stat);
const mkdirp = util.promisify(fs.mkdir);

function throws(code, msg, path) {
	let err = new Error(code + ': ' + msg);
	err.code=code; err.path=path;
	throw err;
}

async function mkdir(str, opts={}) {
	if (process.platform === 'win32' && /[<>:"|?*]/.test(str.replace(path.parse(str).root, ''))) {
		throws('EINVAL', 'invalid characters', str);
	}

	let cwd = path.resolve(opts.cwd || '.');
	let seg, stats, mode = opts.mode || 0o777 & (~process.umask());
	let arr = path.resolve(cwd, path.normalize(str)).replace(cwd, '').split(/[\\\/]+/);

	for (seg of arr) {
		cwd = path.join(cwd, seg);
		if (fs.existsSync(cwd)) {
			stats = await statp(cwd);
			if (!stats.isDirectory()) {
				throws('ENOTDIR', 'not a directory', cwd);
			}
		} else {
			await mkdirp(cwd, mode);
		}
	}

	return cwd;
}

exports.mkdir = mkdir;
