import { promisify } from 'util';
import { stat, mkdir as mkdir$1, existsSync } from 'fs';
import { parse, resolve, normalize, join } from 'path';

const statp = promisify(stat);
const mkdirp = promisify(mkdir$1);

function throws(code, msg, path) {
	let err = new Error(code + ': ' + msg);
	err.code=code; err.path=path;
	throw err;
}

async function mkdir(str, opts={}) {
	if (process.platform === 'win32' && /[<>:"|?*]/.test(str.replace(parse(str).root, ''))) {
		throws('EINVAL', 'invalid characters', str);
	}

	let cwd = resolve(opts.cwd || '.');
	let seg, stats, mode = opts.mode || 0o777 & (~process.umask());
	let arr = resolve(cwd, normalize(str)).replace(cwd, '').split(/[\\\/]+/);

	for (seg of arr) {
		cwd = join(cwd, seg);
		if (existsSync(cwd)) {
			stats = await statp(cwd);
			if (!stats.isDirectory()) {
				throws('ENOTDIR', 'not a directory', cwd);
			}
		} else {
			await mkdirp(cwd, mode);
		}
	}

	return cwd;
}

export { mkdir };
